/***************************************************************************
 *   Copyright (C) 2004 by Sebastian Garcia                                *
 *   s_garcia@speedy.com.ar                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "grp.h"

#include <qsplitter.h>
#include <qlayout.h>

#include <klocale.h>
#include <kaction.h>
#include <kfiledialog.h>
#include <kstandarddirs.h>
#include <kfiledetailview.h>
#include <kfiletreeview.h>

#include <noatun/plugin.h>

#include "grpplaylist.h"
#include "playlistview.h"
#include "selector.h"

// plugin interface
extern "C" Plugin *create_plugin() {
	KGlobal::locale()->insertCatalogue( "grp" );
	GrpPlaylist* grp = new GrpPlaylist;
	return grp;
}

GrpApp::GrpApp( QWidget* p_parent, GrpPlaylist* p_playList, const char* p_name )
	: KMdiMainFrm( p_parent, p_name, KMdi::IDEAlMode ),
	  m_playList( *p_playList ),
		m_playListView( NULL ) {

	m_child = new KMdiChildView( i18n( "Playlist" ), this );

	( new QHBoxLayout( m_child ) )->setAutoAdd( true );
	
	addWindow( m_child );

	m_playListView = new PlaylistView( m_child );

	connect( m_playListView, SIGNAL( itemExecuted( GrpPlaylistItem* ) ), this, SLOT( playItem( GrpPlaylistItem* ) ) );
				
	new KAction(i18n("Add &Files..."), "queue", 0, this, SLOT(addFiles()), actionCollection(), "add_files");

	new KAction(i18n("&Clear playlist"), NULL, 0, m_playListView, SLOT(clearPlaylist()), actionCollection(), "clear_playlist");

	new KAction(i18n("Delete"), "editdelete", Key_Delete, m_playListView, SLOT(deleteSelected()),  actionCollection(), "delete");

	KStdAction::close( this, SLOT( close() ), actionCollection() );

	setXMLFile( "grpplaylistui.rc" );

	createGUI( 0 );

	applyMainWindowSettings( KGlobal::config(), "GRP PlayList" );

	setCaption(i18n( "Playlist" ) );

	showMinimized();

	m_playListView->loadPlaylist( KURL( napp->dirs()->saveLocation( "data", "noatun/" ) + "grpplaylist.xml" ) );

	GroupedSelector *grp = new GroupedSelector( i18n( "My Music" ), "music_sixteenthnote", this, m_playListView, "GroupedSelector" );
	addToolWindow( grp, KDockWidget::DockLeft, getMainDockWidget() );

	FileSelector *files = new FileSelector( i18n( "Files" ), "files", this, m_playListView, "FileSelector" );
	addToolWindow( files, KDockWidget::DockLeft, getMainDockWidget() );

	showNormal();
	hide();
}


GrpApp::~GrpApp() {

	m_playListView->savePlaylist( KURL( napp->dirs()->saveLocation( "data", "noatun/" ) + "grpplaylist.xml" ) );

}

GrpPlaylistItem *GrpApp::getActiveFile(){
	return m_playListView->current();
}

void GrpApp::selectNextFile() {
	m_playListView->selectNext();
}

void GrpApp::selectPreviousFile() {
	m_playListView->selectPrev();
}

void GrpApp::addFiles() {
	KURL::List files=KFileDialog::getOpenURLs( 0, napp->mimeTypes(), this, i18n( "Select File to Add" ) );

	for( KURL::List::Iterator it = files.begin(); it != files.end(); ++it )
		m_playListView->addItem( *it );
}

void GrpApp::addFile( const KURL &p_url, bool p_play ) {
	m_playListView->addItem( p_url, p_play );
}

void GrpApp::clearPlaylist() {
	m_playListView->clearPlaylist();
}

void GrpApp::playItem( GrpPlaylistItem *p_item ) {
	m_playList.setCurrent( p_item, true );
}

bool GrpApp::queryClose() {
	m_playList.hideList();
	return false;
}
