/***************************************************************************
 *   Copyright (C) 2004 by Sebastian Garcia                                *
 *   s_garcia@speedy.com.ar                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#ifndef PLAYLISTVIEW_H
#define PLAYLISTVIEW_H

#include <klistview.h>
#include <kurl.h>

#include <noatun/playlistsaver.h>

class GrpPlaylistItem;

class PlaylistView;

class PlaylistViewItem : public QListViewItem {

public:

	PlaylistViewItem( PlaylistView *parent, const KURL &p_url );

	~PlaylistViewItem();

	const KURL &url() const;
	GrpPlaylistItem *item();

	void setPlaying( bool );

protected:

	virtual void paintCell( QPainter * p, const QColorGroup & cg, int column, int width, int align );

	KURL m_url;
	GrpPlaylistItem *m_grpItem;

	bool m_playing;

};

class PlaylistView : public KListView, protected PlaylistSaver {

	Q_OBJECT

public:

	PlaylistView(QWidget *parent = 0, const char *name = 0);

	~PlaylistView();

	GrpPlaylistItem *current() const;

public slots:

	void addItem( const KURL&, bool = false );
	void addItems( const KURL::List&, bool = false );

	void deleteSelected();
	void clearPlaylist();

	void selectFirst();
	void selectLast();
	void selectNext();
	void selectPrev();

	void loadPlaylist( const KURL& );
	void savePlaylist( const KURL& );

protected slots:

	void executed( QListViewItem*, const QPoint&, int );

signals:

	void itemExecuted( GrpPlaylistItem* );

protected:

	PlaylistViewItem *m_playingItem;

	/* For PlaylistSaver */

protected:

	PlaylistViewItem *m_writedItem;

	void readItem( const QMap< QString, QString > & );
	PlaylistItem writeItem();
};

inline const KURL &PlaylistViewItem::url() const {
	return m_url;
}

inline GrpPlaylistItem *PlaylistViewItem::item() {
	return m_grpItem;
}

#endif
