/***************************************************************************
 *   Copyright (C) 2004 by Sebastian Garcia                                *
 *   s_garcia@speedy.com.ar                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
 
#include <qlayout.h>
#include <qtooltip.h>
#include <qheader.h>
#include <qeventloop.h>
#include <qdom.h>

#include <kdiroperator.h>
#include <kpushbutton.h>
#include <kurlcombobox.h>
#include <kcombobox.h>
#include <kiconloader.h>
#include <klocale.h>
#include <kurl.h>
#include <kmimetype.h>
#include <kfiledialog.h>
#include <kpopupmenu.h>
#include <kdebug.h>
#include <kprogress.h>
#include <kstandarddirs.h>
#include <kmessagebox.h>

#include <noatun/plugin.h>

#include "selector.h"
#include "playlistview.h"

Selector::Selector( const QString &p_caption, const QString &p_icon, QWidget *p_parent, PlaylistView *p_playList, const char *p_name ) 
	: KMdiChildView( p_caption, p_parent, p_name ),
		m_playListView( p_playList ) {

	setCaption( i18n( p_caption ) );
	setIcon( SmallIcon( p_icon ) );

	makeFilePatterns();
}

Selector::~Selector() {

}

void Selector::makeFilePatterns() {
	QStringList mimes = QStringList::split( " ", napp->mimeTypes() );

	for( QStringList::iterator mimeIt = mimes.begin(); mimeIt != mimes.end(); mimeIt++ ) {
		KMimeType::Ptr mime = KMimeType::mimeType( *mimeIt );
		
		QStringList patterns = mime->property( "Patterns" ).toStringList();
	
		for( QStringList::iterator pattIt = patterns.begin(); pattIt != patterns.end(); pattIt++ )
			m_filePatterns += *pattIt + QString( " " );
	}
}

GrpListItemBase::GrpListItemBase( GrpList *p_list ) 
	: QListViewItem( p_list ),
	  m_list( p_list ),
		m_parentGroup( NULL ) {

}

GrpListItemBase::GrpListItemBase( GrpList *p_list, GrpListItemGroup *p_group )
	: QListViewItem( p_group ),
	  m_list( p_list ),
		m_parentGroup( p_group ) {

}

void GrpListItemBase::removeSelected() {

	QValueList< QListViewItem* > items;

	for( QListViewItem *item = firstChild(); item; item = item->nextSibling() )
		items.append( item );

	for( QValueList< QListViewItem* >::Iterator item = items.begin(); item != items.end(); item++ ) {
		if( ( *item )->isSelected() )
			delete *item;
		else
			static_cast< GrpListItemBase* >( *item )->removeSelected();
	}
}

GrpListItemFile::GrpListItemFile( GrpList *p_list, const KURL &p_url, const QMap< QString, QString > &p_prop )
	: GrpListItemBase( p_list ),
	  m_url( p_url ),
	  m_prop( p_prop ) {
	setText( 0, ( p_prop[ "Title" ].isEmpty() ) ? p_url.fileName() : p_prop[ "Title" ] );
}

GrpListItemFile::GrpListItemFile( GrpList *p_list, GrpListItemGroup *p_parent, const KURL &p_url, const QMap< QString, QString > &p_prop )
	: GrpListItemBase( p_list, p_parent ),
	  m_url( p_url ),
	  m_prop( p_prop ) {
	setText( 0, ( p_prop[ "Title" ].isEmpty() ) ? p_url.fileName() : p_prop[ "Title" ] );
}

KURL::List GrpListItemFile::getFiles() const {
	KURL::List urls;
	urls.append( m_url );
	return urls;
}

KURL::List GrpListItemFile::getSelectedFiles() const {
	if( isSelected() )
		return getFiles();
	else
		return KURL::List();
}

GrpListItemBase::List GrpListItemFile::getItems() {
	GrpListItemBase::List items;
	items.append( this );
	return items;
}

GrpListItemBase::List GrpListItemFile::getSelectedItems() {
	GrpListItemBase::List items;
	if( isSelected() )
		items.append( this );
	return items;
}

GrpListItemFile *GrpListItemFile::findFile( const KURL &p_url ) {
	return ( m_url == p_url ) ? this : NULL;
}

int GrpListItemFile::compare( QListViewItem *p_item, int p_col, bool p_asc ) const {

	GrpListItemFile *fileItem;
	if( p_item->rtti() != RTTI_GrpListItemFile )
		return 1;

	fileItem = static_cast< GrpListItemFile* >( p_item );

	if( !m_prop[ "Tracknumber" ].isEmpty() ) {
		if( fileItem->m_prop[ "Tracknumber" ].isEmpty() )
			return -1;
		else
			return ( m_prop[ "Tracknumber" ].toInt() > fileItem->m_prop[ "Tracknumber" ].toInt() ) ? 1 : -1;
	} else {
		if( fileItem->m_prop[ "Tracknumber" ].isEmpty() )
			return QListViewItem::compare( p_item, p_col, p_asc );
		else
			return 1;
	}
	return 0;
}

GrpListItemGroup::GrpListItemGroup( GrpList *p_list, const QString &p_groupText )
	: GrpListItemBase( p_list ),
	  m_groupText( p_groupText ) {
	setText( 0, p_groupText );
	setPixmap( 0, SmallIcon( "folder" ) );
}

GrpListItemGroup::GrpListItemGroup( GrpList *p_list, GrpListItemGroup *p_parent, const QString &p_groupText )
	: GrpListItemBase( p_list, p_parent ),
	  m_groupText( p_groupText ) {
	setText( 0, p_groupText );
	setPixmap( 0, SmallIcon( "folder" ) );
}

KURL::List GrpListItemGroup::getFiles() const {
	KURL::List urls;
	QListViewItem *item = firstChild();
	while( item ) {
		urls += static_cast< GrpListItemBase* >( item )->getFiles();
		item = item->nextSibling();
	}
	return urls;
}

KURL::List GrpListItemGroup::getSelectedFiles() const {
	KURL::List urls;
	QListViewItem *item = firstChild();

	if( isSelected() )
		return getFiles();

	while( item ) {
		urls += static_cast< GrpListItemBase* >( item )->getSelectedFiles();
		item = item->nextSibling();
	}

	return urls;
}

GrpListItemBase::List GrpListItemGroup::getItems() {
	GrpListItemBase::List items;
	QListViewItem *item = firstChild();
	while( item ) {
		items += static_cast< GrpListItemBase* >( item )->getItems();
		item = item->nextSibling();
	}
	return items;
}

GrpListItemBase::List GrpListItemGroup::getSelectedItems() {
	GrpListItemBase::List items;
	QListViewItem *item = firstChild();
	while( item ) {
		items += static_cast< GrpListItemBase* >( item )->getSelectedItems();
		item = item->nextSibling();
	}
	if( isSelected() )
		items += this;
	return items;
}

GrpListItemGroup *GrpListItemGroup::getGroup( const QString &p_groupText ) {
	QListViewItem *item = firstChild();
	while( item ) {
		if( item->rtti() != RTTI_GrpListItemGroup )
			continue;
		if( static_cast< GrpListItemGroup* >( item )->m_groupText == p_groupText )
			return static_cast< GrpListItemGroup* >( item );
		item = item->nextSibling();
	}

	GrpListItemGroup *newGroup = new GrpListItemGroup( m_list, this, p_groupText );
	return newGroup;
}

void GrpListItemGroup::addFileItem( const KURL &p_url, const QMap< QString, QString > &p_prop ) {
	new GrpListItemFile( m_list, this, p_url, p_prop );
}

void GrpListItemGroup::removeItem( GrpListItemBase *p_item ) {
	delete p_item;
	if( !firstChild() ){
		if( getParentGroup() )
			getParentGroup()->removeItem( this );
		else
			m_list->removeItem( this );
	}
}

GrpListItemFile *GrpListItemGroup::findFile( const KURL &p_url ) {
	GrpListItemFile *file = NULL;
	QListViewItem *item = firstChild();
	while( item ) {
		if( ( file = static_cast< GrpListItemBase* >( item )->findFile( p_url ) ) )
			return file;
		item = item->nextSibling();
	}
	return NULL;
}

GrpList::GrpList( QWidget *p_parent, const char *p_name )
	: KListView( p_parent, p_name ),
	  m_progressBar( NULL ) {

	addColumn( "Groups" );
	header()->hide();

	setRootIsDecorated( true );
	setSelectionModeExt( Extended );

}

void GrpList::setGroups( const QStringList &p_groups ) {
	m_groups = p_groups;
}

void GrpList::setProgressBar( KProgress *p_progress ) {
	m_progressBar = p_progress;
}

KURL::List GrpList::getFiles() const {
	KURL::List urls;
	QListViewItem *item = firstChild();

	while( item ) {
		urls += static_cast< GrpListItemBase* >( item )->getFiles();
		item = item->nextSibling();
	}

	return urls;
}

KURL::List GrpList::getSelectedFiles() const {

	KURL::List urls;
	QListViewItem *item = firstChild();

	while( item ) {
		urls += static_cast< GrpListItemBase* >( item )->getSelectedFiles();
		item = item->nextSibling();
	}

	return urls;
}

GrpListItemBase::List GrpList::getItems() {
	GrpListItemBase::List items;
	QListViewItem *item = firstChild();
	while( item ) {
		items += static_cast< GrpListItemBase* >( item )->getItems();
		item = item->nextSibling();
	}
	return items;
}

GrpListItemBase::List GrpList::getSelectedItems() {
	GrpListItemBase::List items;
	QListViewItem *item = firstChild();
	while( item ) {
		items += static_cast< GrpListItemBase* >( item )->getSelectedItems();
		item = item->nextSibling();
	}
	return items;
}

GrpListItemFile *GrpList::findFile( const KURL &p_url ) {
	GrpListItemFile *file = NULL;
	QListViewItem *item = firstChild();
	while( item ) {
		if( ( file = static_cast< GrpListItemBase* >( item )->findFile( p_url ) ) )
			return file;
		item = item->nextSibling();
	}
	return NULL;
}

void GrpList::addFile( const KURL &p_url ) {
	QMap<QString, QString> prop;

	KFileMetaInfo info( p_url );

	if( !info.isValid() )
		return;

/*	QStringList keys = info.supportedKeys();

	for( QStringList::ConstIterator it = keys.begin(); it != keys.end(); it++ )
		prop.insert( *it, info.item( *it ).string() );*/
	prop.insert( "url", p_url.url() );
	prop.insert( "Title", info.item( "Title" ).string().stripWhiteSpace() );
	prop.insert( "Album", info.item( "Album" ).string().stripWhiteSpace() );
	prop.insert( "Artist", info.item( "Artist" ).string().stripWhiteSpace() );
	prop.insert( "Genre", info.item( "Genre" ).string().stripWhiteSpace() );
	prop.insert( "Tracknumber", info.item( "Tracknumber" ).string().stripWhiteSpace() );

	addFile( p_url, prop );
}

void GrpList::addFile( const KURL &p_url, const QMap<QString, QString> &p_prop ) {
	GrpListItemGroup *group = NULL;
	GrpListItemFile *file = NULL;
	QString groupText;

	if( ( file = findFile( p_url ) ) ){
		if( file->getParentGroup() )
			file->getParentGroup()->removeItem( file );
		else
			removeItem( file );
	}

	for( QStringList::ConstIterator it = m_groups.begin(); it != m_groups.end(); it++ ) {
		groupText = p_prop[ *it ].stripWhiteSpace();
		if( groupText.isEmpty() )
			groupText = i18n( "Unknown " + *it );
		group = group ? group->getGroup( groupText ) : getGroup( groupText );
	}
	
	if( group ) {
		group->addFileItem( p_url, p_prop );
		group->sortChildItems( 0, true );
	} else {
		addFileItem( p_url, p_prop );
		sort();
	}
}
void GrpList::addFiles( const KURL::List &p_urls ) {
	int i = 0;
	
	if( m_progressBar ) {
		m_progressBar->show();
		m_progressBar->setTotalSteps( p_urls.count() );
	}
	
	for( KURL::List::ConstIterator url = p_urls.begin(); url != p_urls.end(); url++ ) {
		if( m_progressBar )
			m_progressBar->setProgress( ++i );
		addFile( *url );
		if( ( i % 15 ) == 0 )
			KApplication::kApplication()->eventLoop()->processEvents( QEventLoop::ExcludeUserInput );
	}
	
	if( m_progressBar )
		m_progressBar->hide();
}

void GrpList::removeSelected() {

	QValueList< QListViewItem* > items;

	for( QListViewItem *item = firstChild(); item; item = item->nextSibling() )
		items.append( item );
	
	for( QValueList< QListViewItem* >::Iterator item = items.begin(); item != items.end(); item++ ) {
		if( ( *item )->isSelected() )
			delete *item;
		else
			static_cast< GrpListItemBase* >( *item )->removeSelected();
	}
}

void GrpList::refresh() {
	KURL::List urls = getFiles();
	clearList();
	addFiles( urls );
}

void GrpList::clearList() {
	KListView::clear();
}

void GrpList::saveList( const KURL &p_url ) {
	QDomDocument doc( "GRPList" );

	QDomElement rootElem = doc.createElement( "GRPList" );
	doc.appendChild( rootElem );
	rootElem.setAttribute( "version", "1.0" );

	GrpListItemBase::List items = getItems();

	for( GrpListItemBase::List::ConstIterator item = items.begin(); item != items.end(); item++ ) {
		if( ( *item )->rtti() != RTTI_GrpListItemFile )
			continue;

		QDomElement itemElem = doc.createElement( "item" );
		rootElem.appendChild( itemElem );

		QMap< QString, QString > props = static_cast< GrpListItemFile* >( *item )->getProperties();
		for( QMap< QString, QString >::ConstIterator prop = props.begin(); prop != props.end(); prop++ )
			itemElem.setAttribute( prop.key().utf8(), prop.data().utf8() );
	}

	QFile file( p_url.path() );

	if( ! file.open( IO_WriteOnly ) ) {
		kdDebug() << "*** Error opening the file \"" << p_url.path() << "\": " << file.errorString() << endl;
//		KMessageBox::error( 0, i18n( "Error open the file %1 for writing." ).arg( p_url.path() ), i18n( "Error" ) );
		return;
	}

	QTextStream stream( &file );
	stream << doc.toString();
	file.close();
}

void GrpList::readList( const KURL &p_url ) {
	QDomDocument doc( "GRPList" );
	QString errMsg;
	QFile file( p_url.path() );

	if( !file.open( IO_ReadOnly ) )
		return;

	if( !doc.setContent( &file, &errMsg ) ) {
		kdDebug() << "Parse error: \"" << errMsg << "\"" << endl;
		file.close();
		return;
	}

	file.close();

	QDomElement rootElem = doc.documentElement();
	QDomNode itemNode = rootElem.firstChild();

	while( ! itemNode.isNull() ) {
		QDomElement itemElem = itemNode.toElement();
		if( itemElem.isNull() )
			continue;

		QMap< QString, QString > props;
		props.insert( "url", itemElem.attribute( "url" ) );
		props.insert( "Title", itemElem.attribute( "Title" ) );
		props.insert( "Album", itemElem.attribute( "Album" ) );
		props.insert( "Artist", itemElem.attribute( "Artist" ) );
		props.insert( "Genre", itemElem.attribute( "Genre" ) );
		props.insert( "Tracknumber", itemElem.attribute( "Tracknumber" ) );
		addFile( KURL( props[ "url" ] ), props );

		itemNode = itemNode.nextSibling();
	}
}

GrpListItemGroup *GrpList::getGroup( const QString &p_groupText ) {
	QListViewItem *item = firstChild();
	while( item ) {
		if( item->rtti() != RTTI_GrpListItemGroup )
			continue;
		if( static_cast< GrpListItemGroup* >( item )->m_groupText == p_groupText )
			return static_cast< GrpListItemGroup* >( item );
		item = item->nextSibling();
	}

	GrpListItemGroup *newGroup = new GrpListItemGroup( this, p_groupText );
	return newGroup;
}

void GrpList::addFileItem( const KURL &p_url, const QMap< QString, QString > &p_prop ) {
	new GrpListItemFile( this, p_url, p_prop );
}

void GrpList::removeItem( GrpListItemBase *p_item ) {
	delete p_item;
}

GroupedSelector::GroupedSelector( const QString &p_caption, const QString &p_icon, QWidget *p_parent, PlaylistView *p_playList, const char *p_name )
	: Selector( p_caption, p_icon, p_parent, p_playList, p_name ) {

	QVBoxLayout *mainLayout = new QVBoxLayout( this );

	QHBoxLayout *buttonsLayout = new QHBoxLayout();

	KPushButton *addFilesButton = new KPushButton( this, "addFilesButton" );
	addFilesButton->setPixmap( SmallIcon( "fileopen" ) );
	QToolTip::add( addFilesButton, i18n( "Add files" ) );
	addFilesButton->setFlat( true );
	buttonsLayout->addWidget( addFilesButton );

	KPushButton *addDirButton = new KPushButton( this, "addDirButton" );
	addDirButton->setPixmap( SmallIcon( "filefind" ) );
	QToolTip::add( addDirButton, i18n( "Add directory (recursive)" ) );
	addDirButton->setFlat( true );
	buttonsLayout->addWidget( addDirButton );

	KPushButton *deleteButton = new KPushButton( this, "deleteButton" );
	deleteButton->setPixmap( SmallIcon( "remove" ) );
	QToolTip::add( deleteButton, i18n( "Delete" ) );
	deleteButton->setFlat( true );
	buttonsLayout->addWidget( deleteButton );

	KPushButton *reloadButton = new KPushButton( this, "reloadButton" );
	reloadButton->setPixmap( SmallIcon( "reload" ) );
	QToolTip::add( reloadButton, i18n( "Reload" ) );
	reloadButton->setFlat( true );
	buttonsLayout->addWidget( reloadButton );

	QSpacerItem *spacer = new QSpacerItem( 100, 21, QSizePolicy::Expanding, QSizePolicy::Minimum );
	buttonsLayout->addItem( spacer );

	KPushButton *addButton = new KPushButton( this, "addButton" );
	addButton->setPixmap( SmallIcon( "edit_add" ) );
	QToolTip::add( addButton, i18n( "Add selected files to play list" ) );
	addButton->setFlat( true );
	buttonsLayout->addWidget( addButton );

	KPushButton *playButton = new KPushButton( this, "playButton" );
	playButton->setPixmap( SmallIcon( "player_play" ) );
	QToolTip::add( playButton, i18n( "Play selected files" ) );
	playButton->setFlat( true );
	buttonsLayout->addWidget( playButton );

	mainLayout->addLayout( buttonsLayout );

	m_grpList = new GrpList( this, "m_grpList" );
	mainLayout->addWidget( m_grpList );

	QStringList groups;
	groups.append( "Artist" );
	groups.append( "Album" );
	m_grpList->setGroups( groups );

	KProgress *progress = new KProgress( this );
	mainLayout->addWidget( progress );
	m_grpList->setProgressBar( progress );
	progress->hide();

	connect( addFilesButton, SIGNAL( clicked() ), this, SLOT( addFiles() ) );
	connect( addDirButton, SIGNAL( clicked() ), this, SLOT( addDir() ) );
	connect( deleteButton, SIGNAL( clicked() ), m_grpList, SLOT( removeSelected() ) );
	connect( reloadButton, SIGNAL( clicked() ), m_grpList, SLOT( refresh() ) );

	connect( addButton, SIGNAL( clicked() ), this, SLOT( addSelectedFiles() ) );
	connect( playButton, SIGNAL( clicked() ), this, SLOT( playSelectedFiles() ) );

	connect( m_grpList, SIGNAL( contextMenu( KListView*, QListViewItem*, const QPoint& ) ), this, SLOT( contextMenu( KListView*, QListViewItem*, const QPoint& ) ) );
	connect( m_grpList, SIGNAL( doubleClicked( QListViewItem*, const QPoint&, int ) ), this, SLOT( doubleClicked( QListViewItem*, const QPoint&, int ) ) );

	m_grpList->readList( KURL( napp->dirs()->saveLocation( "data", "noatun/" ) + "grpplist.xml" ) );

}

GroupedSelector::~ GroupedSelector() {
	m_grpList->saveList( KURL( napp->dirs()->saveLocation( "data", "noatun/" ) + "grpplist.xml" ) );
}

void GroupedSelector::addFiles() {
	KURL::List files = KFileDialog::getOpenURLs( QString::null, napp->mimeTypes(), this, i18n( "Select File to Add" ) );

	for( KURL::List::ConstIterator file = files.begin(); file != files.end(); file++ )
		m_grpList->addFile( *file );
}

void GroupedSelector::addDir() {

	KURL::List urls, addUrls;

	QString path = KFileDialog::getExistingDirectory( QString::null, this, i18n( "Select folder to add" ) );

	if( path.isEmpty() )
		return;

	urls = getFiles( QDir( path ) );

	for( KURL::List::Iterator url = urls.begin(); url != urls.end(); url++ )
		if( !m_grpList->findFile( *url ) )
			addUrls += *url;

	m_grpList->addFiles( addUrls );
}

void GroupedSelector::addSelectedFiles() {
	m_playListView->addItems( m_grpList->getSelectedFiles(), false );
	m_playListView->setFocus();
}

void GroupedSelector::playSelectedFiles() {
	m_playListView->addItems( m_grpList->getSelectedFiles(), true );
	m_playListView->setFocus();
}

void GroupedSelector::contextMenu( KListView*, QListViewItem *p_item, const QPoint &p_point ) {

	if( !p_item )
		return;

	KPopupMenu menu( this );

	menu.insertItem( i18n( "Add item" ), this, SLOT( addSelectedFiles() ) );
	menu.insertItem( i18n( "Play item" ), this, SLOT( playSelectedFiles() ) );
	menu.insertItem( i18n( "Remove item" ), m_grpList, SLOT( removeSelected() ) );

	menu.exec( p_point );
}

void GroupedSelector::doubleClicked( QListViewItem *p_item, const QPoint&, int ) {
	if( !p_item )
		return;

	m_playListView->clearPlaylist();
	m_playListView->addItems( static_cast< GrpListItemBase* >( p_item )->getFiles(), true );
	m_playListView->setFocus();
}

KURL::List GroupedSelector::getFiles( const QDir &p_dir ) {
	KURL url;
	KURL::List urls;
	QStringList subDirs = p_dir.entryList( QDir::Dirs );

	for( QStringList::ConstIterator subDir = subDirs.begin(); subDir != subDirs.end(); subDir++ )
		if( ( *subDir )[0] != '.' )
			urls += getFiles( QDir( p_dir.canonicalPath() + QDir::separator() + *subDir ) );

	QStringList files = p_dir.entryList( m_filePatterns );

	for( QStringList::ConstIterator file = files.begin(); file != files.end(); file++ ){
		url.setPath( p_dir.canonicalPath() + QDir::separator() + *file );
		urls += url;
	}

	return urls;
}

FileSelector::FileSelector( const QString &p_caption, const QString &p_icon, QWidget *p_parent, PlaylistView *p_playList, const char *p_name )
	: Selector( p_caption, p_icon, p_parent, p_playList, p_name ) {

	QVBoxLayout *mainLayout = new QVBoxLayout( this );

	QVBoxLayout *layout = new QVBoxLayout();

	QHBoxLayout *buttonsLayout = new QHBoxLayout();

	KPushButton *parentButton = new KPushButton( this, "parentButton" );
	parentButton->setSizePolicy( QSizePolicy( QSizePolicy::Fixed, QSizePolicy::Fixed ) );
	parentButton->setPixmap( SmallIcon( "up" ) );
	QToolTip::add( parentButton, i18n( "Up one level" ) );
	buttonsLayout->addWidget( parentButton );

	KPushButton *backButton = new KPushButton( this, "backButton" );
	backButton->setSizePolicy( QSizePolicy( QSizePolicy::Fixed, QSizePolicy::Fixed ) );
	backButton->setPixmap( SmallIcon( "back" ) );
	QToolTip::add( backButton, i18n( "back" ) );
	buttonsLayout->addWidget( backButton );

	KPushButton *forwardButton = new KPushButton( this, "forwardButton" );
	forwardButton->setSizePolicy( QSizePolicy( QSizePolicy::Fixed, QSizePolicy::Fixed ) );
	forwardButton->setPixmap( SmallIcon( "forward" ) );
	QToolTip::add( forwardButton, i18n( "forward" ) );
	buttonsLayout->addWidget( forwardButton );

	KPushButton *homeButton = new KPushButton( this, "homeButton" );
	homeButton->setSizePolicy( QSizePolicy( QSizePolicy::Fixed, QSizePolicy::Fixed ) );
	homeButton->setPixmap( SmallIcon( "gohome" ) );
	QToolTip::add( homeButton, i18n( "Home directory" ) );
	buttonsLayout->addWidget( homeButton );

	QSpacerItem *buttonsSpacer = new QSpacerItem( 81, 21, QSizePolicy::Expanding, QSizePolicy::Minimum );
	buttonsLayout->addItem( buttonsSpacer );

	KPushButton *addButton = new KPushButton( this, "addButton" );
	addButton->setPixmap( SmallIcon( "edit_add" ) );
	QToolTip::add( addButton, i18n( "Add selected files" ) );
	buttonsLayout->addWidget( addButton );

	KPushButton *playButton = new KPushButton( this, "playButton" );
	playButton->setPixmap( SmallIcon( "player_play" ) );
	QToolTip::add( playButton, i18n( "Play selected files" ) );
	buttonsLayout->addWidget( playButton );

	layout->addLayout( buttonsLayout );

	m_currentDirCombo = new KURLComboBox( KURLComboBox::Directories, true, this, "m_currentDirCombo" );
	layout->addWidget( m_currentDirCombo );

	m_dirOperator = new KDirOperator( QString::null, this, "m_dirOperator" );
	m_dirOperator->setView( KFile::Simple );
	m_dirOperator->setOnlyDoubleClickSelectsFiles( true );
	m_dirOperator->setMode( KFile::Files );
	layout->addWidget( m_dirOperator );

	QHBoxLayout *filterLayout = new QHBoxLayout();

	QLabel *filterIcon = new QLabel( this, "applyFilterButton" );
	filterIcon->setPixmap( BarIcon( "filter" ) );
	filterLayout->addWidget( filterIcon );

	KHistoryCombo *filterEdit = new KHistoryCombo( this, "filterEdit" );
	filterEdit->setSizePolicy( QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Fixed ) );
	filterLayout->addWidget( filterEdit );

	layout->addLayout( filterLayout );
	mainLayout->addLayout( layout );

	connect( filterEdit, SIGNAL( activated( const QString& ) ), this, SLOT( filterChanged( const QString& ) ) );
	connect( filterEdit, SIGNAL( returnPressed( const QString& ) ), filterEdit, SLOT( addToHistory( const QString& ) ) );

	connect( parentButton, SIGNAL( clicked() ), m_dirOperator, SLOT( cdUp() ) );
	connect( backButton, SIGNAL( clicked() ), m_dirOperator, SLOT( back() ) );
	connect( forwardButton, SIGNAL( clicked() ), m_dirOperator, SLOT( forward() ) );
	connect( homeButton, SIGNAL( clicked() ), m_dirOperator, SLOT( home() ) );

	connect( addButton, SIGNAL( clicked() ), this, SLOT( addSelectedFiles() ) );
	connect( playButton, SIGNAL( clicked() ), this, SLOT( playSelectedFiles() ) );

	connect( m_currentDirCombo, SIGNAL( urlActivated( const KURL& ) ), this, SLOT( setURL( const KURL& ) ) );
	connect( m_currentDirCombo, SIGNAL( returnPressed( const QString& ) ), this, SLOT( setURL( const QString& ) ) );

	connect( m_dirOperator, SIGNAL( urlEntered( const KURL& ) ), this, SLOT( urlEntered( const KURL& ) ) );
	connect( m_dirOperator, SIGNAL( fileSelected( const KFileItem* ) ), this, SLOT( fileSelected( const KFileItem* ) ) );

	urlEntered( m_dirOperator->url() );

	filterEdit->setCurrentText( m_filePatterns );
	filterEdit->addToHistory( m_filePatterns );
	filterChanged( m_filePatterns );
}

FileSelector::~ FileSelector() {

}

void FileSelector::setURL( const KURL &p_url ) {
	m_dirOperator->setURL( p_url, true );
}

void FileSelector::setURL( const QString &p_url ) {
	m_dirOperator->setURL( KURL( p_url ) , true );
}

void FileSelector::addSelectedFiles() {
	const KFileItemList *list = m_dirOperator->selectedItems();
	KURL::List urlList;

	if ( !list )
		return;

	for( KFileItemList::ConstIterator it = list->begin(); it != list->end(); it++ ) {
		if( (*it)->isFile() )
			urlList += (*it)->url();
	}

	m_playListView->addItems( urlList, false );
}

void FileSelector::playSelectedFiles() {
	const KFileItemList *list = m_dirOperator->selectedItems();
	KURL::List urlList;

	if ( !list )
		return;

	for( KFileItemList::ConstIterator it = list->begin(); it != list->end(); it++ ) {
		if( (*it)->isFile() )
			urlList += (*it)->url();
	}

	m_playListView->addItems( urlList, true );
}

void FileSelector::filterChanged( const QString &p_filter ) {
	m_dirOperator->setNameFilter( p_filter );
	m_dirOperator->rereadDir();
}

void FileSelector::urlEntered( const KURL &p_url ) {

	QStringList urlList;

	m_currentDirCombo->removeURL( p_url );

	urlList = m_currentDirCombo->urls();
	urlList.prepend( p_url.url() );
	while( urlList.count() >= (uint)m_currentDirCombo->maxItems() )
		urlList.remove( urlList.last() );
	
	m_currentDirCombo->setURLs( urlList );
}

void FileSelector::fileSelected( const KFileItem *p_file ) {
	m_playListView->addItem( p_file->url() );
}

